ISpy = LibStub("AceAddon-3.0"):NewAddon("ISpy", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")

local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("ISpy", true)
local ISpy_MESSAGECOLOR = "ffffff00"

ISpy.DeathLog = {}

function ISpy:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("ISpyDB", ISpy_DEFAULTS, "Default")
	
	self.options = ISpy_Options
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	self.options.args.profiles.inline = true
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("ISpy", self.options, "ispy")
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("ISpy", "ISpy")
	
	self:RegisterChatCommand("ispy", "ChatCommand")
end

function ISpy:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self.options.args.otheroptions.args.enable.hidden = true
	self:RawHook("SetItemRef","ParseLink",true)
	print("|c"..ISpy_MESSAGECOLOR..L["ISpy"].."|r: "..L["Enabled"].." - "..L["Type \"/ispy\" for options or \"/ispy help\" for console commands"])
end

function ISpy:OnDisable()
	self.options.args.otheroptions.args.enable.hidden = false
	print("|c"..ISpy_MESSAGECOLOR..L["ISpy"].."|r: "..L["Disabled"])
end

function ISpy:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
	else
		if input:trim() == "help" then input = "" end
		LibStub("AceConfigCmd-3.0").HandleCommand(self, "ispy", "ISpy", input)
	end
end

function ISpy:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local outmsg = false
	local srcSpell, destSpell
	local outSrc, outDst
	local Deathlink

	if self.db.profile.onlyingroup and GetNumRaidMembers() == 0 and GetNumPartyMembers() == 0 then return end
	
	local timestamp, eventType, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags, spellID, spellName, _, extraskillID, extraskillname = ...;
	local Amount

	if eventType == "RANGE_DAMAGE" or eventType == "SPELL_PERIODIC_DAMAGE" then eventType = "SPELL_DAMAGE" end
	
	if eventType ~= "SPELL_INTERRUPT" and eventType ~= "SPELL_STOLEN" and eventType ~= "SPELL_DISPELL" and eventType ~= "SPELL_AURA_REMOVED" and eventType ~= "SPELL_AURA_BROKEN" and eventType ~= "SPELL_AURA_BROKEN_SPELL" and eventType ~= "UNIT_DIED" and eventType ~= "SPELL_DAMAGE" and eventType ~= "SWING_DAMAGE" and eventType ~= "ENVIRONMENTAL_DAMAGE" and eventType ~= "UNIT_DESTROYED" then return end
	
	if eventType == "UNIT_DESTROYED" then ISpy.DeathLog[destGUID] = nil end

	if eventType == "UNIT_DIED" and not self.db.profile.death then return end
	if eventType == "SPELL_DAMAGE" and not self.db.profile.death then return end
	if eventType == "SWING_DAMAGE" and not self.db.profile.death then return end
	if eventType == "ENVIRONMENTAL_DAMAGE" and not self.db.profile.death then return end
	
	if eventType == "SPELL_INTERRUPT" and not self.db.profile.interrupt then return end
	if eventType == "SPELL_STOLEN" and not self.db.profile.steal then return end
	if eventType == "SPELL_AURA_REMOVED" and not self.db.profile.auraremoved then return end
	if eventType == "SPELL_AURA_BROKEN" and not self.db.profile.aurabroken then return end
	if eventType == "SPELL_AURA_BROKEN_SPELL" and not self.db.profile.aurabrokenspell then return end
	
	if eventType == "SPELL_DISPELL" then
		local enemy = UnitIsEnemy(sourceGUID, destGUID)
		if self.db.profile.debuff_friendly and enemy then return end
		if self.db.profile.debuff_hostile and not enemy then return end
	end
	if not self.db.profile.pvp then
		for i,v in pairs(self.db.profile.excludedzones) do
			if GetRealZoneText() == v then
				return
			end
		end
	end

	if eventType == "SWING_DAMAGE" then extraskillID = spellID end
	
	if eventType == "SWING_DAMAGE" or eventType == "SPELL_DAMAGE" or eventType == "ENVIRONMENTAL_DAMAGE" then
		ISpy.DeathLog[destGUID] = ISpy.DeathLog[destGUID] or {}
		for i = 3,2,-1 do
			ISpy.DeathLog[destGUID][i] = ISpy.DeathLog[destGUID][i] or {}
			ISpy.DeathLog[destGUID][i-1] = ISpy.DeathLog[destGUID][i-1] or {}
			
			ISpy.DeathLog[destGUID][i].timestamp = ISpy.DeathLog[destGUID][i-1].timestamp or ""
			ISpy.DeathLog[destGUID][i].src = ISpy.DeathLog[destGUID][i-1].src or ""
			ISpy.DeathLog[destGUID][i].spellID = ISpy.DeathLog[destGUID][i-1].spellID or ""
			ISpy.DeathLog[destGUID][i].Amount = ISpy.DeathLog[destGUID][i-1].Amount or ""
		end
		ISpy.DeathLog[destGUID][1].timestamp = timestamp
		ISpy.DeathLog[destGUID][1].src = sourceName or "Environmental"
		ISpy.DeathLog[destGUID][1].spellID = (eventType == "SWING_DAMAGE") and L["Melee"] or spellID
		ISpy.DeathLog[destGUID][1].Amount = tonumber(extraskillID) and extraskillID or 0
		return
	end
	
	if eventType == "UNIT_DIED" then
		outmsg = L["Death"]
		if ISpy.DeathLog[destGUID] then
			spellID = ISpy.DeathLog[destGUID][1].spellID or "<Unknown>"
			sourceName = ISpy.DeathLog[destGUID][1].src or "<Unknown>"
			Amount = ISpy.DeathLog[destGUID][1].Amount or "<none>"
			Deathlink = ""
			for i = 1,3 do
				Deathlink = Deathlink .. ":" .. ISpy.DeathLog[destGUID][i].timestamp .. ":" .. ISpy.DeathLog[destGUID][i].src .. ":" .. ISpy.DeathLog[destGUID][i].spellID .. ":" .. ISpy.DeathLog[destGUID][i].Amount
			end
		else
			--spellID = nil
			--sourceName = nil
			return
		end
	end
	
	local grouped = UnitPlayerOrPetInParty(sourceName) or UnitPlayerOrPetInRaid(sourceName) or UnitPlayerOrPetInParty(destName) or UnitPlayerOrPetInRaid(destName) or UnitInParty(sourceName) or UnitInParty(destName)
	
	if self.db.profile.onlygroup and not grouped then
		return
	end
	
	if eventType == "SPELL_INTERRUPT" then outmsg = L["Interrupt"] end
	if eventType == "SPELL_STOLEN" then outmsg = L["Stolen"] end
	if eventType == "SPELL_DISPEL" then outmsg = L["Dispell"] end
	if eventType == "SPELL_AURA_REMOVED" then outmsg = L["Removed"] end
	if eventType == "SPELL_AURA_BROKEN" then outmsg = L["Broken"] end
	if eventType == "SPELL_AURA_BROKEN_SPELL" then outmsg = L["Broken Spell"] end

	if outmsg then
		local outmsg_channel, outmsg_private
		if tonumber(spellID) then
			srcSpell = GetSpellLink(spellID)
		else
			srcSpell = spellID and "[" .. spellID .. "]" or "[" .. L["Unknown"] .. "]"
		end
		
		if eventType == "SPELL_AURA_REMOVED" or eventType == "SPELL_AURA_BROKEN" then
			destSpell = srcSpell
			srcSpell = extraskillID
		else
			if extraskillID then destSpell = GetSpellLink(extraskillID) else destSpell = "" end
		end
		
		if srcSpell ~= "" then srcSpell = " " .. srcSpell .. (Amount and " " .. Amount .. " |HISPY:" .. destGUID .. ":" .. destName .. Deathlink .. "|h|cff71d5ff[...]|r|h" or "") end
		if destSpell ~= "" then destSpell = " " .. destSpell end
		
		outSrc = (sourceName and srcSpell) and sourceName ..  srcSpell or ""
		outDst = (destName and destSpell) and destName .. destSpell .. " " .. L["by"] .. " " or ""
		
		outmsg_channel = outmsg..": " .. outDst .. outSrc
		
		outSrc = (sourceName and srcSpell) and self:ColourName(sourceName) .. srcSpell or ""
		outDst = (destName and destSpell) and self:ColourName(destName) .. destSpell .. " " .. L["by"] .. " " or ""
				
		outmsg_private = "|c"..ISpy_MESSAGECOLOR..outmsg.."|r: " .. outDst .. outSrc
		
		
		if self.db.profile.private then print(outmsg_private) end
		if self.db.profile.say then SendChatMessage(outmsg_channel, "SAY") end
		if self.db.profile.yell then SendChatMessage(outmsg_channel, "YELL") end
		if self.db.profile.guild then SendChatMessage(outmsg_channel, "GUILD") end
		if self.db.profile.officer then SendChatMessage(outmsg_channel, "OFFICER") end
		
		if self.db.profile.battleground and UnitInBattleground("player") then
			SendChatMessage(outmsg_channel, "BATTLEGROUND")
		end
		
		if self.db.profile.whisper and self.db.profile.playername ~= "" then
			SendChatMessage(outmsg_channel, "WHISPER", nil, self.db.profile.playername)
		end
		
		if self.db.profile.channel then
			local index = GetChannelName(self.db.profile.channelname)
			if index > 0 then
				SendChatMessage(outmsg_channel, "CHANNEL", nil, index);
			end
		end
		
		if GetNumRaidMembers() > 0 or GetNumPartyMembers() > 0 then
			if self.db.profile.raidwarning and (IsRaidOfficer() or IsRaidLeader()) and GetNumRaidMembers() > 0 then
				SendChatMessage(outmsg_channel, "RAID_WARNING")
			elseif self.db.profile.raid and GetNumRaidMembers() > 0 then
				SendChatMessage(outmsg_channel, "RAID")
			elseif self.db.profile.party and GetNumPartyMembers() > 0 then
				SendChatMessage(outmsg_channel, "PARTY")
			end
		end
	end
	
	if eventType == "UNIT_DIED" then 
		ISpy.DeathLog[destGUID] = nil
		for i, v in pairs(ISpy.DeathLog) do
			if not UnitExists(i) then ISpy.DeathLog[i] = nil end
		end
	end
end

function ISpy:ColourName(istring)
	if UnitPlayerOrPetInParty(istring) or UnitPlayerOrPetInRaid(istring) or UnitInParty(istring) then
		return "|cFF00FF00" .. istring .. "|r"
	else
		return "|cFFFF0000" .. istring .. "|r"
	end
end

function ISpy:ParseLink(link, text, button, chatframe)
	local linktype, id, name, time1, src1, spell1, amount1, time2, src2, spell2, amount2, time3, src3, spell3, amount3 = strsplit(":", link)
	if linktype == "ISPY" then
		ShowUIPanel(ItemRefTooltip);
		if ( not ItemRefTooltip:IsShown() ) then
			ItemRefTooltip:SetOwner(UIParent, "ANCHOR_PRESERVE");
		end
		
		if time2 == "" then time2 = nil end
		if time3 == "" then time3 = nil end
				
		ItemRefTooltip:ClearLines()
		ItemRefTooltip:AddLine(name.."'s Death")
		ItemRefTooltip:AddLine("Time of Death: " .. date("%c",time1))
		ItemRefTooltip:AddDoubleLine( format("%06.3f",time1 - time1) .. " " .. src1 .. " " .. (GetSpellLink(spell1) or spell1), amount1)
		if time2 and (time1-time2) < 30 then ItemRefTooltip:AddDoubleLine(format("%06.3f",time1 - time2) .. " " .. src2 .. " " .. (GetSpellLink(spell2) or spell2), amount2) end
		if time3 and (time1-time3) < 30 then ItemRefTooltip:AddDoubleLine(format("%06.3f",time1 - time3) .. " " .. src3 .. " " .. (GetSpellLink(spell3) or spell3), amount3) end
		ItemRefTooltip:Show()
	else
		return ISpy.hooks["SetItemRef"](link, text, button, chatframe)
	end
end